//---------------------------------------------------------------------------------------
// File          : ztec_com_sft_reg.v
// Project       : FPGA Thermal Design Reserarch
// Created       : 2016/12/01
// Last Author   : $Author:$
// Last modified : $Date:$
// Revision      : $Revision:$
//---------------------------------------------------------------------------------------
// Copyright (c) 2016 by Zuken Tec Inc.
//---------------------------------------------------------------------------------------

// shift register(FF)  with enable, clear (common module)
module ztec_com_sft_reg ( ck, rx, cl, en, id, od );

    parameter p_width = 2;   // bit width   ( width >= 1 )
    parameter p_cycle = 2;   // shift cycle ( cycle >= 1 )
    parameter P_DL    = 1;   // FF delay

    input                ck; // clock
    input                rx; // async reset (active low)
    input                cl; // shift data clear
    input                en; // shift enable
    input  [p_width-1:0] id; // input data
    output [p_width-1:0] od; // output data

    integer              ii; // loop variable (for loop)

    reg    [p_width-1:0] ff_reg [0:p_cycle-1]; // shift register(FF)
    
    always @( posedge ck or negedge rx )begin
        if( ~rx )begin
            for( ii = 0; ii < p_cycle; ii = ii + 1 )begin
                ff_reg[ii] <= {p_width{1'd0}};
            end
        end
        else if( cl )begin
            for( ii = 0; ii < p_cycle; ii = ii + 1 )begin
                ff_reg[ii] <= #P_DL {p_width{1'd0}};
            end
        end
        else if( en )begin
            ff_reg[0] <= #P_DL id;
            for( ii = 1; ii < p_cycle; ii = ii + 1 )begin
                ff_reg[ii] <= #P_DL ff_reg[ii-1];
            end
        end
    end

    assign od = ff_reg[p_cycle-1];
    
endmodule

/* RTL instantiation template
 
 case1              bit sft
  ztec_com_sft_reg #( 8, 1 ) aaa_d1 ( .ck( clk_in ), .rx( rst_x ), .cl( ff_cl ), .en( ff_en ), .id( bbb[7:0] ), .od( bbb_d1[7:0] ) );
 
 case2               bit sft
  ztec_com_sft_reg #( 4, 2 ) bbb_d
    (
     .ck ( clk_in          ), // _i_
     .rx ( rst_x           ), // _i_
     .cl ( ff_cl           ), // _i_
     .en ( ff_en           ), // _i_
     .id ( ccc[3:0]        ), // _i_
     .od ( ccc_d2[3:0]     )  // _o_
     );

 */
