//---------------------------------------------------------------------------------------
// File          : ztec_core.v
// Project       : FPGA Thermal Design Reserarch
// Created       : 2016/12/01
// Last Author   : $Author:$
// Last modified : $Date:$
// Revision      : $Revision:$
//---------------------------------------------------------------------------------------
// Copyright (c) 2016 by Zuken Tec Inc.
//---------------------------------------------------------------------------------------

// core module
module ztec_core
    ( 
       clk
      ,rstx
      ,tgl_sel
      ,out_start
      ,out_dt
      );

    parameter P_FF_NUM = 10;  // Shift FF Num
    parameter P_DL     = 1;   // FF delay
                              
    input          clk;       // clock
    input          rstx;      // async reset (active low)
    input  [1:0]   tgl_sel;   // toggle select
    output         out_start; // toggle start (for debug)
    output         out_dt;    // output data


    // toggle start (for debug : FF Num Check)
    reg            ff_out_start;
    
    always @( posedge clk or negedge rstx )begin
        if( ~rstx ) ff_out_start <= 1'b0;
        else        ff_out_start <= #P_DL 1'b1;
    end

    assign out_start = ff_out_start;
    
    // data_toggle signal
    wire [2:0] count;
    ztec_com_count #(3) cnt
        (
         .ck ( clk               ), // _i_
         .rx ( rstx              ), // _i_
         .cl ( 1'b0              ), // _i_
         .en ( 1'b1              ), // _i_
         .ct ( count             )  // _o_
         );

    wire sft_dt;
    assign sft_dt = ( tgl_sel == 2'b00 ) ? 1'b0     : // Not Toggle
                    ( tgl_sel == 2'b01 ) ? count[2] : // Toggle  25%
                    ( tgl_sel == 2'b10 ) ? count[1] : // Toggle  50%
                                           count[0] ; // Toggle 100%

    // shift register
    ztec_com_sft_reg #( 1, P_FF_NUM ) sft
        (
         .ck ( clk               ), // _i_
         .rx ( rstx              ), // _i_
         .cl ( 1'b0              ), // _i_
         .en ( 1'b1              ), // _i_
         .id ( sft_dt            ), // _i_
         .od ( out_dt            )  // _o_
         );
    
endmodule

