//---------------------------------------------------------------------------------------
// File          : ztec_top_xilinx.v
// Project       : FPGA Thermal Design Reserarch
// Created       : 2016/12/01
// Last Author   : $Author:$
// Last modified : $Date:$
// Revision      : $Revision:$
//---------------------------------------------------------------------------------------
// Copyright (c) 2016 by Zuken Tec Inc.
//---------------------------------------------------------------------------------------

// top module
module ztec_top_xilinx
    ( 
       clk_in
      ,rstx
      ,clk_sel
      ,tgl_sel
      ,out_start
      ,out_start2
      ,out_dt
      ,out_dt2

      ,vsns5v0_p
      ,vsns5v0_n
      ,isns5v0_p
      ,isns5v0_n
      ,isns0v95_p
      ,isns0v95_n
      );

    parameter P_INST_NUM = 5;    // ztec_core(ShiftRegModule) Num

    //  Artix7 (XC7A35T) all FF = 41600FF -> 5200CLB * 8FF
 
    parameter P_FF_NUM = 39500;      // Shift FF Num (caseA : FF Max FPGA)
//  parameter P_FF_NUM = 20000;      // Shift FF Num (caseB : FF 1/2 FPGA)
//  parameter P_FF_NUM = 10000;      // Shift FF Num (caseC : FF 1/4 FPGA)
//  parameter P_FF_NUM = P_INST_NUM; // Shift FF Num (caseD : FF Min FPGA)
 
    parameter P_DL     = 1;      // FF delay
                              
    input          clk_in;       // board clock
    input          rstx;         // async reset (active low)
    input  [1:0]   clk_sel;      // clock select
    input  [1:0]   tgl_sel;      // toggle select
    output         out_start;    // toggle start (for debug) LED pin
    output         out_start2;   // toggle start (for debug) Port pin
    output         out_dt;       // output data LED pin
    output         out_dt2;      // output data Port pin

    input          vsns5v0_p;    // Arty 5v sens for XADC
    input          vsns5v0_n;
    input          isns5v0_p;    // Arty 5v current sens for XADC
    input          isns5v0_n;
    input          isns0v95_p;   // Arty Core current sens for XADC
    input          isns0v95_n;

    // Arty 100MHz clk in BUFG
    BUFG BUFG
      (
         .I  (clk_in     ), // 1-bit input: Clock input
         .O  (clk_in_buf )  // 1-bit output: Clock output
       );

     // XADC
    xadc_wiz_0 xadc
        (
         .dclk_in     (clk_in_buf ), // input wire dclk_in
         .vp_in       (1'b0       ), // input wire vp_in
         .vn_in       (1'b0       ), // input wire vn_in
         .vauxp1      (vsns5v0_p  ), // input wire vauxp1
         .vauxn1      (vsns5v0_n  ), // input wire vauxn1
         .vauxp9      (isns5v0_p  ), // input wire vauxp9
         .vauxn9      (isns5v0_n  ), // input wire vauxn9
         .vauxp10     (isns0v95_p ), // input wire vauxp10
         .vauxn10     (isns0v95_n ), // input wire vauxn10
         .channel_out (           ), // output wire [4 : 0] channel_out
         .eoc_out     (           ), // output wire eoc_out
         .alarm_out   (           ), // output wire alarm_out
         .eos_out     (           ), // output wire eos_out
         .busy_out    (           )  // output wire busy_out
         );

    // Generate Clock : PLL (xilinx cell MMCM)
    wire           clk0;         //  10.000MHz
    wire           clk1;         // 100.000MHz
    wire           clk2;         // 275.000MHz
    wire           clk3;         // 366.667MHz

    clk_wiz_0 XILINX_PLLE2          // MMCM
        (
         // Clock in ports
         .clk_in1  (clk_in_buf ),     // input clk_in1
         // Clock out ports
         .clk_out1 (clk0       ),     // output clk_out1
         .clk_out2 (clk1       ),     // output clk_out2
         .clk_out3 (clk2       ),     // output clk_out3
         .clk_out4 (clk3       ),     // output clk_out4
         // Status and control signals
         .resetn   (rstx       )      // input resetn
         );

    // Clock Select : BUFGMUX (xilinx cell)
    wire           clk01;
    BUFGMUX BUFGMUX01
        (
         .I0 (clk0        ), // 1-bit input: Clock input (S=0)
         .I1 (clk1        ), // 1-bit input: Clock input (S=1)
         .S  (clk_sel[0]  ), // 1-bit input: Clock select
         .O  (clk01       )  // 1-bit output: Clock output
         );

    wire           clk23;
    BUFGMUX BUFGMUX23
        (
         .I0 (clk2        ), // 1-bit input: Clock input (S=0)
         .I1 (clk3        ), // 1-bit input: Clock input (S=1)
         .S  (clk_sel[0]  ), // 1-bit input: Clock select
         .O  (clk23       )  // 1-bit output: Clock output
         );

    wire           clk0123;
    BUFGMUX BUFGMUX0123
        (
         .I0 (clk01       ), // 1-bit input: Clock input (S=0)
         .I1 (clk23       ), // 1-bit input: Clock input (S=1)
         .S  (clk_sel[1]  ), // 1-bit input: Clock select
         .O  (clk0123     )  // 1-bit output: Clock output
         );

    wire [P_INST_NUM-1:0]    out_start_tmp;
    wire [P_INST_NUM-1:0]    out_dt_tmp;

    genvar         j;
    generate
        // zcore instance loop
        for( j=0; j<P_INST_NUM; j=j+1 ) begin : zcore

            // vivado setting
            (* DONT_TOUCH = "yes" *)
            ztec_core
               #( P_FF_NUM/P_INST_NUM )
            ztec_core
               (
                .clk        (clk0123                        ),
                .rstx       (rstx                           ),
                .tgl_sel    (tgl_sel                        ),
                .out_start  (out_start_tmp[j]               ),
                .out_dt     (out_dt_tmp[j]                  )
                );
        end
    endgenerate

    // Output (LED)
    assign out_start  = |out_start_tmp;
    assign out_dt     = |out_dt_tmp;

    // Output (Port)
    assign out_start2 = |out_start_tmp;
    assign out_dt2    = |out_dt_tmp;

endmodule

